/* model.c
   ___________________________________________________

   Model File:  polynomial.A.model

   Date:  Mon May 21 14:01:22 2001

   Created by:  "mod v4.0"
    -- a model preprocessor by Don Maszle
   ___________________________________________________

   Copyright (c) 1993-96 by F. Bois & D. Maszle.  All rights reserved.

  Model calculations for compartmental model:

    0 States:

    4 Outputs:
      V_out1 	-> 0.0
      V_out2 	-> 0.0
      U_out2 	-> 0.0
      U_out3 	-> 0.0

    0 Inputs:

    5 Parameters:
      U 	:= 1
      y_S 	:= 1
      M_yS 	:= 1
      theta 	:= 1
      alpha 	:= 1
*/


#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <float.h>
#include "modelu.h"
#include "random.h"
#include "yourcode.h"

/*----- Indices to Global Variables */

/* Model variables: States and other outputs */
#define ID_V_out1	0x0000
#define ID_V_out2	0x0001
#define ID_U_out2	0x0002
#define ID_U_out3	0x0003

/* Inputs */

/* Parameters */
#define ID_U	0x0004
#define ID_y_S	0x0005
#define ID_M_yS	0x0006
#define ID_theta	0x0007
#define ID_alpha	0x0008

/*----- Global Variables */

  /* For export.  Keep track of who we are. */
char szModelDescFilename[] = "polynomial.A.model";
char szModelSourceFilename[] = __FILE__;
char szModelGenAndVersion[] = "mod v4.0";

  /* Externs */
extern BOOL vbModelReinitd;

  /* Model Dimensions */
int vnStates	= 0;
int vnOutputs	= 4;
int vnModelVars	= 4;
int vnInputs	= 0;
int vnParms	= 5;

  /* States and Outputs*/
double vrgModelVars[4];

  /* Inputs */
IFN vrgInputs[1];

/* Parameters */
  double U;
  double y_S;
  double M_yS;
  double theta;
  double alpha;


/*----- Global Variable Map */
VMMAPSTRCT vrgvmGlo[] = {

    {"V_out1",	(PVOID) &vrgModelVars[ID_V_out1], 	ID_OUTPUT | ID_V_out1},
    {"V_out2",	(PVOID) &vrgModelVars[ID_V_out2], 	ID_OUTPUT | ID_V_out2},
    {"U_out2",	(PVOID) &vrgModelVars[ID_U_out2], 	ID_OUTPUT | ID_U_out2},
    {"U_out3",	(PVOID) &vrgModelVars[ID_U_out3], 	ID_OUTPUT | ID_U_out3},


    {"U",	(PVOID) &U, 	ID_PARM | ID_U},
    {"y_S",	(PVOID) &y_S, 	ID_PARM | ID_y_S},
    {"M_yS",	(PVOID) &M_yS, 	ID_PARM | ID_M_yS},
    {"theta",	(PVOID) &theta, 	ID_PARM | ID_theta},
    {"alpha",	(PVOID) &alpha, 	ID_PARM | ID_alpha},
    {"",	NULL,	0} /* End flag */
};  /* vrgpvmGlo[] */

/* InitModel

   Should be called to initialize model variables at
   the beginning of experiment before reading
   variants from the simulation spec file.
*/

void InitModel(void)
{
/*-- Initialize things in the order that they appear in
     model definition file so that dependencies are
     handled correctly. */

  vrgModelVars[ID_V_out1] = 0.0;
  vrgModelVars[ID_V_out2] = 0.0;
  vrgModelVars[ID_U_out2] = 0.0;
  vrgModelVars[ID_U_out3] = 0.0;
  U = 1;
  y_S = 1;
  M_yS = 1;
  theta = 1;
  alpha = 1;

  vbModelReinitd = TRUE;  /* Flag is initialized */
} /* InitModel */

/* Dynamics section */
void CalcDeriv (double  rgModelVars[], double  rgDerivs[], PDOUBLE pdTime)
{
  /* Local variables */

  CalcInputs (pdTime);  /* Get new input vals */

} /* CalcDeriv */


/* Model scaling */
void ScaleModel (void)
{
  /* Local variables */

} /* ScaleModel */


/* Jacobian calculation */
void CalcJacob (double rgModelVars[], double *rgdSave[], PDOUBLE pdTime)
{
  /* Local variables */

} /* CalcJacob */


/* Outputs calculation */
void CalcOutputs (double  rgModelVars[], double  rgDerivs[], PDOUBLE pdTime)
{
  /* Local variables */
  double tmp;

  rgModelVars[ID_V_out1] = y_S / theta ;

  rgModelVars[ID_U_out2] = U ;
  tmp = ( y_S - U ) / ( theta + alpha * U ) ;
  rgModelVars[ID_V_out2] = ( tmp > 0 ? tmp : 1E-30 ) ;

  rgModelVars[ID_U_out3] = y_S ;

}  /* CalcOutputs */


